#include "traceAsyncWebsocket.hpp"
#include <Chronos.hpp>


int TraceAsyncWebsocket::modulo(int a, int b)
{
    int r = a % b;

    return ((r < 0) ? r + b : r);
}

TraceLine & TraceAsyncWebsocket::tLine(const uint16_t index)
{
    int start = _lines_index - _lines_count;
    int idx = modulo(start + index, _lines_count);

    return (_lines[idx]);
}

void TraceAsyncWebsocket::print(uint8_t severity, const char * buffer, int length)
{
    char lin[MAX_LINE_LEN + 1];
    unsigned int lineptr = 0;

    while ((0 != *buffer) && (lineptr < (sizeof(lin) - 1))) {
        if (*buffer > 0x1f)
            lin[lineptr] = *buffer;
        else
            lin[lineptr] = '?';
        ++lineptr;
        ++buffer;
    }
    lin[lineptr] = 0; // ukoncime retezec

    TraceLine line(severity, lin, lineptr);

    _lines[_lines_index++] = line;
    _lines_index %= MAX_TRACE_LINES;
    if (_lines_count < MAX_TRACE_LINES) {
        ++_lines_count;
    }
    _modified = _modified + 1;
}

void TraceAsyncWebsocket::loop()
{

    if (_wss) {
        // je definovany webovy soket
        if (_modified) {
            // mame nejakou zmenu
            if (_wss->count() != 0) {
                // mame klienty
                String log;

                _modified = 0; // rusime pozadavek na odeslani novych dat
                if (log.reserve((MAX_TRACE_LINES * MAX_LINE_LEN) + (MAX_TRACE_LINES * 50))) {
                    log = F("{\"type\":\"trace\",\"data\":[");
                    dumpJSON(log);
                    log.concat(F("]}"));
                    if (_wss->availableForWriteAll())
                        _wss->textAll(log);
                } else if (_wss->availableForWriteAll())
                    _wss->textAll(F("{\"type\":\"trace\",\"text\":\"Memory error\"}"));
            } else
                _modified = 0; // zadny pripojeny klient - po pripojeni stejne musime vyzadat stav, takze ted muzeme modifikaci klidne ignorovat
        }
    }
}

void TraceAsyncWebsocket::init()
{

    Chronos::attachMs(TRACEWS_CHECK_INTERVAL, this);
}

String TraceAsyncWebsocket::_getText(const char * buffer)
{
    String res;

    res.reserve(MAX_LINE_LEN * 4);

    while (0 != *buffer) {
        switch (*buffer) { // uprava escape sekvenci pro JSON/HTML
            case '"':
                res.concat(F("\\\""));
                break;

            case '\\':
                res.concat(F("\\\\"));
                break;

            case '/':
                res.concat(F("\\/"));
                break;

            case '<':
                res.concat(F("&lt;"));
                break;

            case '>':
                res.concat(F("&gt;"));
                break;

            default:
                if (*buffer < 128)
                    res.concat(*buffer);
                else {
                    char chr[3];
                    res.concat(F("\\u00"));
                    sprintf_P(chr, PSTR("%02X"), *buffer);
                    res.concat(chr);
                }
                break;
        }
        ++buffer;
    }
    return res;
}

void TraceAsyncWebsocket::dumpJSON(String & str)
{

    for (int i = 0; i < _lines_count; i++) {
        TraceLine line = tLine(i);

        if (0 != i)
            str.concat(F(","));
        str.concat(F("{\"t\":"));
        str.concat(line._time);
        str.concat(F(",\"s\":"));
        str.concat(line._severity);
        str.concat(F(",\"d\":\""));
        str.concat(_getText(line._text));
        str.concat(F("\"}"));
    }
}

void TraceAsyncWebsocket::print(uint8_t severity, const __FlashStringHelper * fmt, ...)
{
    char buffer[MAX_LINE_LEN + 1];
    va_list args;

    va_start(args, fmt);
    int length = vsnprintf_P(buffer, sizeof(buffer), (const char *) fmt, args);
    va_end(args);

    print(severity, buffer, length);
}

void TraceAsyncWebsocket::print(uint8_t severity, const char * fmt, ...)
{
    char buffer[MAX_LINE_LEN + 1];
    va_list args;

    va_start(args, fmt);
    int length = vsnprintf(buffer, sizeof(buffer), fmt, args);
    va_end(args);

    print(severity, buffer, length);
}

char TraceAsyncWebsocket::hexascii(uint8_t n)
{

    n &= 0xf;
    if (n > 9)
        return n + ('A' - 10);
    else
        return n + '0';
}

void TraceAsyncWebsocket::dump(uint8_t severity, const char * prefix, const uint8_t * address, size_t size)
{
    char buffer[MAX_LINE_LEN + 1 + 3];
    int idx = 0;

    if (prefix)
        idx = snprintf_P(buffer, MAX_LINE_LEN, PSTR("%s"), prefix);

    while ((idx < MAX_LINE_LEN) && size) {
        buffer[idx] = hexascii(*address >> 4);
        ++idx;
        buffer[idx] = hexascii(*address);
        ++idx;
        buffer[idx] = 0x20;
        ++idx;
        ++address;
        --size;
    }
    buffer[idx] = 0;
    print(severity, buffer, idx);
}

void TraceAsyncWebsocket::dump(uint8_t severity, const __FlashStringHelper * prefix, const uint8_t * address, size_t size)
{
    char buffer[MAX_LINE_LEN + 1 + 3];
    int idx = 0;

    if (prefix) {
        char dummy[64];

        strcpy_P(dummy, (const char *) prefix);
        idx = snprintf_P(buffer, MAX_LINE_LEN, PSTR("%s"), dummy);
    }

    while ((idx < MAX_LINE_LEN) && size) {
        buffer[idx] = hexascii(*address >> 4);
        ++idx;
        buffer[idx] = hexascii(*address);
        ++idx;
        buffer[idx] = 0x20;
        ++idx;
        ++address;
        --size;
    }
    buffer[idx] = 0;
    print(severity, buffer, idx);
}

void TraceAsyncWebsocket::forceupdate()
{

    _modified = _modified + 1; // vynutime odeslani informaci
}

// EOF
