/**
 * @file traceWebsocket.h
 * @author Pavel Brychta, http://www.xpablo.cz
 *
 * Copyright (c) 2016-24 Pavel Brychta. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#pragma once

#include "trace.h"
#include <Arduino.h>
#include <Chronos.hpp>
#include <ESPAsyncWebServer.h>

#define TRACEWS_CHECK_INTERVAL 200 // interval [ms], po kterem je testovano odesilani stopare

#define MAX_TRACE_LINES        15
#define MAX_LINE_LEN           50

struct TraceLine {
    char _text[MAX_LINE_LEN + 1] = {0};
    uint32_t _time{};
    uint8_t _severity{};

    TraceLine(uint8_t severity, const char * str, uint16_t len) {
        strncpy(_text, str, sizeof(_text));
        _text[MAX_LINE_LEN] = 0;
        _time = millis();
        _severity = severity;
    }

    TraceLine() = default;
};

class TraceAsyncWebsocket final : public Trace, public Loop {
  protected:
    TraceLine _lines[MAX_TRACE_LINES];
    uint16_t _lines_index = 0;
    uint16_t _lines_count = 0;
    volatile int _modified = 0;
    AsyncWebSocket * _wss = nullptr; // webovy soket pro trasovani

    static int modulo(int a, int b);

    TraceLine & tLine(uint16_t index);

    static char hexascii(uint8_t n);

    static String _getText(const char * buffer);

    void dumpJSON(String & str);

    void print(uint8_t severity, const char * buffer, int length);

  public:
    explicit TraceAsyncWebsocket(AsyncWebSocket * srv) : _wss(srv) {};

    void init() override;

    void print(uint8_t severity, const __FlashStringHelper * fmt, ...) override;

    void print(uint8_t severity, const char * fmt, ...) override;

    void dump(uint8_t severity, const char * prefix, const uint8_t * address, size_t size) override;

    void dump(uint8_t severity, const __FlashStringHelper * prefix, const uint8_t * address, size_t size) override;

    void loop() override;

    void forceupdate() override;
};
